package org.ziggurat.fenix.common.database.hibernate;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.LockMode;
import org.hibernate.criterion.Example;
import org.hibernate.exception.ConstraintViolationException;

/**
 * Implements the generic CRUD data access operations using Hibernate APIs.
 * <p>
 * To write a DAO, subclass and parameterize this class with your persistent
 * class. Of course, assuming that you have a traditional 1:1 appraoch for
 * Entity:DAO design.
 * <p>
 * You have to inject a current Hibernate <tt>Session</tt> to use a DAO.
 * Otherwise, this generic implementation will use
 * <tt>HibernateUtil.getSessionFactory()</tt> to obtain the current
 * <tt>Session</tt>.
 * 
 * @see HibernateDAOFactory
 * 
 * @author Christian Bauer
 */
public abstract class GenericHibernateRepository<T, ID extends Serializable> {

	private static Logger log = Logger.getLogger("fenix");
	
	private Class<T> persistentClass;

	@SuppressWarnings("unchecked")
	public GenericHibernateRepository() {
		this.persistentClass = (Class<T>) ((ParameterizedType) getClass()
				.getGenericSuperclass()).getActualTypeArguments()[0];
	}

	public Class<T> getPersistentClass() {
		return persistentClass;
	}

	@SuppressWarnings("unchecked")
	public T findById(ID id, boolean lock) {
		log.trace("Find by id: "+ id.getClass() + "-" + id);
		T entity;
		if (lock)
			entity = (T) HibernateUtil.currentSession().load(getPersistentClass(), id,
					LockMode.UPGRADE);
		else
			entity = (T) HibernateUtil.currentSession().load(getPersistentClass(), id);

		return entity;
	}

	@SuppressWarnings("unchecked")
	public List<T> findAll() {
		log.trace("findAll" + getPersistentClass().toString());
		Criteria crit = HibernateUtil.currentSession().createCriteria(getPersistentClass());
		return crit.list();
	}
	
	@SuppressWarnings("unchecked")
	public List<T> findAllPaginated(int start, int count) {
		log.trace("findAllPaginated" + getPersistentClass().toString());
		Criteria crit = HibernateUtil.currentSession().createCriteria(getPersistentClass());
		crit.setFirstResult(start);
		crit.setMaxResults(count);
		return crit.list();
	}

	@SuppressWarnings("unchecked")
	public List<T> findByExample(T exampleInstance, String... excludeProperty) {
		Criteria crit = HibernateUtil.currentSession().createCriteria(getPersistentClass());
		Example example = Example.create(exampleInstance);
		for (String exclude : excludeProperty) {
			example.excludeProperty(exclude);
		}
		crit.add(example);
		return crit.list();
	}

	@SuppressWarnings("unchecked")
	public T makePersistent(T entity) {
		log.info("Persistiendo " + entity.toString());
		HibernateUtil.currentSession().beginTransaction();
		try {
			HibernateUtil.currentSession().saveOrUpdate(entity);
			HibernateUtil.currentSession().getTransaction().commit();
			log.info("OK");
		} catch (ConstraintViolationException cve) {
			HibernateUtil.currentSession().getTransaction().rollback();
			HibernateUtil.currentSession().clear();
			throw cve;
		}
		return entity;
	}

	public void makeTransient(T entity) {
		HibernateUtil.currentSession().beginTransaction();
		HibernateUtil.currentSession().delete(entity);
		HibernateUtil.currentSession().getTransaction().commit();
	}

}
