package org.ziggurat.fenix.common.database.hibernate;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.time.DateUtils;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.ziggurat.fenix.common.modelo.proveedores.CCProveedor;
import org.ziggurat.fenix.common.modelo.proveedores.CategoriaDeProveedor;
import org.ziggurat.fenix.common.modelo.proveedores.CompraAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.PagoAProveedorInmutable;
import org.ziggurat.fenix.common.modelo.proveedores.Proveedor;
import org.ziggurat.fenix.common.modelo.proveedores.TransaccionConProveedor;
import org.ziggurat.fenix.common.util.MoneyHelper;

public class HibProveedorRepository extends GenericHibernateRepository<Proveedor, Long> {

	
	public List<TransaccionConProveedor> getTransaccionesDeProveedor(Proveedor prov) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria criPagos = session.createCriteria(PagoAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.eq("eliminado", false));
		Criteria criCompras = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.eq("eliminado", false));
		List<TransaccionConProveedor> results = criPagos.list();
		results.addAll(criCompras.list());
		Collections.sort(results);
		return results;
	}
	
	/**
	 * Retorna la lista de transacciones de un proveedor (incluyendo pagos y compras) entre
	 * dos fechas y ordenada por fecha de manera descendente (primero las transacciones mas recientes)
	 * @param prov
	 * @param desde
	 * @param hasta
	 * @return
	 */
	public List<TransaccionConProveedor> getTransaccionesDeProveedorEntreFechas(Proveedor prov, final Date desde, final Date hasta) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria criPagos = session.createCriteria(PagoAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.between("fecha", desde, hasta));
		Criteria criCompras = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.between("fecha", desde, hasta));
		List<TransaccionConProveedor> results = criPagos.list();
		results.addAll(criCompras.list());
		Collections.sort(results);
		return results;		
	}
	
	/**
	 * Retorna las ultimas N transacciones de cada tipo.
	 * @param prov
	 * @param cantidad
	 * @return
	 */
	public List<TransaccionConProveedor> getUltimasTransaccionesDeProveedor(Proveedor prov, int cantidad) {
		Calendar cal = Calendar.getInstance();
		Date desde = DateUtils.add(cal.getTime(), Calendar.MONTH, -3);
		Date hasta = DateUtils.add(cal.getTime(),Calendar.YEAR, 80);
		List<TransaccionConProveedor> results = getTransaccionesDeProveedorEntreFechas(prov, desde, hasta);
		if (results.size() > cantidad) {
			return getTransaccionesDeProveedorEntreFechas(prov, results.get(results.size() - cantidad).getFecha(), hasta);
		} else {
			return results;
		}
	}
	
	/**
	 * Retorna la lista de pagos de un proveedor en orden descendente
	 * (las transaccioes mas recientes en primer lugar).
	 * @param Proveedor
	 * @return List<PagoAProveedorInmutable>
	 */
	public List<PagoAProveedorInmutable> getPagosDeProveedor(Proveedor prov) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria criTrans = session.createCriteria(PagoAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))			
			.add(Restrictions.eq("eliminado", false))
			.addOrder(Order.asc("fecha"));
		List<PagoAProveedorInmutable> transacciones = criTrans.list();
		return transacciones; 
	}
	
	/**
	 * Retorna la lista de pagos de un proveedor entre dos fechas
	 * @param prov
	 * @param desde
	 * @param hasta
	 * @return
	 */
	public List<PagoAProveedorInmutable> getPagosDeProveedorEntreFechas(Proveedor prov, Date desde, Date hasta) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria cri = session.createCriteria(PagoAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.between("fecha", desde, hasta))
			.addOrder(Order.desc("fecha"));
		return cri.list();
	}
	
	/**
	 * Retorna una lista de los pagos con cheque realizado a un proveedor
	 * @param prov
	 * @return
	 */
	public List<PagoAProveedorInmutable> getPagosDeProveedorConCheques(Proveedor prov) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria cri = session.createCriteria(PagoAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.isNotNull("cheque"))
			.add(Restrictions.eq("eliminado", false))
			.addOrder(Order.asc("fecha"));
		return cri.list();
	}
	
	/**
	 * Retorna las compras(facturas) a un proveedor
	 * @param prov
	 * @return
	 */
	public List<CompraAProveedorInmutable> getComprasAProveedor(Proveedor prov) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria criTrans = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.eq("eliminado", false))
			.addOrder(Order.asc("fecha"));
		List<CompraAProveedorInmutable> transacciones = criTrans.list();
		return transacciones;
	}
	
	/**
	 * Retorna la lista de compras a un proveedor entre dos fechas
	 * @param prov
	 * @param desde
	 * @param hasta
	 * @return
	 */
	public List<CompraAProveedorInmutable> getComprasAProveedorEntreFechas(Proveedor prov, Date desde, Date hasta) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria criTrans = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.eq("eliminado", false))
			.add(Restrictions.between("fecha", desde, hasta))
			.addOrder(Order.desc("fecha"));
		List<CompraAProveedorInmutable> transacciones = criTrans.list();
		return transacciones;
	}
	
	
	public CCProveedor getCuentaCorrienteConUltimosMovimientos(Proveedor prov) {
		List<TransaccionConProveedor> transacciones = getUltimasTransaccionesDeProveedor(prov, 9);
		BigDecimal saldoInicial = BigDecimal.ZERO;
		if (transacciones.size() > 0) {
			saldoInicial = getSaldoDeProveedorHasta(prov, transacciones.get(0).getFecha());
		}		
		CCProveedor cc = new CCProveedor(transacciones, saldoInicial);
		return cc;
	} 
	
	/**
	 * Retotna la cuenta corriente del proveedor entre dos fechas
	 * @param prov
	 * @param desde
	 * @param hasta
	 * @return CCProveedor
	 */
	public CCProveedor getCuentaCorrienteEntreFechas(Proveedor prov, Date desde, Date hasta) {
		List<TransaccionConProveedor> transacciones = getTransaccionesDeProveedorEntreFechas(prov, desde, hasta);
		BigDecimal saldoInicial = getSaldoDeProveedorHasta(prov, desde);
		CCProveedor cc = new CCProveedor(transacciones, saldoInicial);
		return cc;
	}
	
	/**
	 * Retorna la cuenta corriente con todas las transacciones del proveedor
	 * sin restricciones de fechas.
	 * @param prov
	 * @return
	 */
	public CCProveedor getCuentaCorriente(Proveedor prov) {
		List<TransaccionConProveedor> transacciones = getTransaccionesDeProveedor(prov);
		CCProveedor cc = new CCProveedor(transacciones, BigDecimal.ZERO);
		return cc;
	}
	
	/*
	public BigDecimal getSaldoDeProveedorHasta(Proveedor prov, Date hasta) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria criCompras = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.le("fecha", hasta))
			.add(Restrictions.eq("eliminado", false));
		List<CompraAProveedorInmutable> compras = criCompras.list();
		BigDecimal suma = BigDecimal.ZERO;
		for (CompraAProveedorInmutable compra : compras) {
			suma = MoneyHelper.restar(suma, compra.getMonto());
		}
		Criteria criPagos = session.createCriteria(PagoAProveedorInmutable.class)
		.add(Restrictions.eq("proveedor", prov))
		.add(Restrictions.le("fecha", hasta))
		.add(Restrictions.eq("eliminado", false));
		List<PagoAProveedorInmutable> pagos = criPagos.list();
		for (PagoAProveedorInmutable pago : pagos) {
			suma = MoneyHelper.sumar(suma, pago.getMonto());
		}
		return suma;
	}*/
	
	public BigDecimal getSaldoDeProveedorHasta(Proveedor prov, Date hasta) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria criCompras = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.lt("fecha", hasta))
			.add(Restrictions.eq("eliminado", false))
			.setProjection(Projections.sum("total"));
		BigDecimal compras = (BigDecimal) criCompras.uniqueResult();
		Criteria criPagos = session.createCriteria(PagoAProveedorInmutable.class)
		.add(Restrictions.eq("proveedor", prov))
		.add(Restrictions.lt("fecha", hasta))
		.add(Restrictions.eq("eliminado", false))
		.setProjection(Projections.sum("monto"));
		BigDecimal pagos = (BigDecimal) criPagos.uniqueResult();
		if (compras == null) {
			compras = BigDecimal.ZERO;
		}
		if (pagos == null) {
			pagos = BigDecimal.ZERO;
		}
		return MoneyHelper.restar(pagos, compras);		
	}
	/*
	public BigDecimal getSaldoDeProveedor(Proveedor prov) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria criCompras = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.eq("eliminado", false));
		List<CompraAProveedorInmutable> compras = criCompras.list();
		BigDecimal suma = BigDecimal.ZERO;
		for (CompraAProveedorInmutable compra : compras) {
			suma = MoneyHelper.restar(suma, compra.getMonto());
		}
		Criteria criPagos = session.createCriteria(PagoAProveedorInmutable.class)
		.add(Restrictions.eq("proveedor", prov))
		.add(Restrictions.eq("eliminado", false));
		List<PagoAProveedorInmutable> pagos = criPagos.list();
		for (PagoAProveedorInmutable pago : pagos) {
			suma = MoneyHelper.sumar(suma, pago.getMonto());
		}
		return suma;
	}*/
	
	public BigDecimal getSaldoDeProveedor(Proveedor prov) {
		Session session = HibernateUtil.currentSession();
		session.beginTransaction();
		Criteria criCompras = session.createCriteria(CompraAProveedorInmutable.class)
			.add(Restrictions.eq("proveedor", prov))
			.add(Restrictions.eq("eliminado", false))
			.setProjection(Projections.sum("total"));
		BigDecimal compras = (BigDecimal) criCompras.uniqueResult();
		Criteria criPagos = session.createCriteria(PagoAProveedorInmutable.class)
		.add(Restrictions.eq("proveedor", prov))
		.add(Restrictions.eq("eliminado", false))
		.setProjection(Projections.sum("monto"));
		BigDecimal pagos = (BigDecimal) criPagos.uniqueResult();
		if (compras == null) {
			compras = BigDecimal.ZERO;
		}
		if (pagos == null) {
			pagos = BigDecimal.ZERO;
		}
		return MoneyHelper.restar(pagos, compras);
	}
	
	public List<Proveedor> getProveedoresPorNombre(String nombre) {
		Session session = HibernateUtil.currentSession();
		String nombreLike = "%" + nombre + "%";
		Criteria cri = session.createCriteria(Proveedor.class)
			.add(Restrictions.ilike("nombre", nombreLike));
		return cri.list();
	}

	@Override
	public List<Proveedor> findAll() {
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(Proveedor.class)
			.addOrder(Order.asc("nombre"))
			.list();
	}
	
	public List<Proveedor> findProveedoresDeudores() {
		List<Proveedor> proveedores = findAll();
		List<Proveedor> result = new ArrayList<Proveedor>();
		for (Proveedor prov : proveedores) {
			if (getSaldoDeProveedor(prov).compareTo(BigDecimal.ZERO) < 0) {
				result.add(prov);
			}
		}
		return result;
	}

    //TODO testear
    public List<Proveedor> getProveedoresConDeudaPorMasDeXDias(int dias) {
        List<Proveedor> proveedores = findAll();
		List<Proveedor> result = new ArrayList<Proveedor>();
		for (Proveedor prov : proveedores) {
			if (getSaldoDeProveedor(prov).compareTo(BigDecimal.ZERO) < 0) {
				List<PagoAProveedorInmutable> pagos = getPagosDeProveedorEntreFechas(prov, DateUtils.add(new Date(), Calendar.DAY_OF_MONTH, dias), new Date());
                if (pagos.size() == 0) {
                    result.add(prov);
                }
			}
		}
		return result;
    }
	
	/**
	 * Retorna una pagina de la lista de proveedores
	 * @param start
	 * @param count
	 * @return
	 */
	public List<Proveedor> findListaDeProveedoresPage(int start, int count) {
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(Proveedor.class)
			.addOrder(Order.asc("nombre"))
			.setFirstResult(start)
			.setMaxResults(count)
			.list();
	}
	
	/**
	 * Retorna una pagina de la lista de proveedores cuyo
	 * nombre coincide con el string de busqueda
	 * @param start
	 * @param count
	 * @return
	 */
	public List<Proveedor> findListaDeProveedoresPage(int start, int count, String busqueda) {
		String nombreLike = "%" + busqueda + "%";
		Session session = HibernateUtil.currentSession();
		return session.createCriteria(Proveedor.class)
			.add(Restrictions.ilike("nombre", nombreLike))
			.addOrder(Order.asc("nombre"))
			.setFirstResult(start)
			.setMaxResults(count)
			.list();
	}
	
	public int countProveedoresFiltrados(String busqueda) {
		String nombreLike = "%" + busqueda + "%";
		Session session = HibernateUtil.currentSession();
		return (Integer)session.createCriteria(Proveedor.class)
			.add(Restrictions.ilike("nombre", nombreLike))
			.setProjection(Projections.rowCount())
			.uniqueResult();
	}
	
	
	public int countProveedores() {
		Session session = HibernateUtil.currentSession();
		return (Integer) session.createCriteria(Proveedor.class)
			.setProjection(Projections.rowCount())
			.uniqueResult();
	}

	/*
	public List<CategoriaDeProveedor> getCategorias() {
		Session session = HibernateUtil.currentSession();
		Criteria cri = session.createCriteria(CategoriaDeProveedor.class);			
		return cri.list();
	}
	*/
	
	public List<Proveedor> getProveedoresPorCategoria(CategoriaDeProveedor unaCategoria) {
		Session session = HibernateUtil.currentSession();
		Criteria cri = session.createCriteria(Proveedor.class)
			.add(Restrictions.eq("categoria", unaCategoria))
			.addOrder(Order.asc("nombre"));
		return cri.list();
	}

}
