package org.ziggurat.fenix.common.modelo.comprobantes.factura;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.ziggurat.fenix.common.modelo.clientes.Cliente;

@Entity
@Table(name = "factura")
public class FacturaInmutable implements Comparable<FacturaInmutable> , Serializable {

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private long id;
	
	@ManyToOne( cascade = {CascadeType.PERSIST, CascadeType.MERGE} )
	private final Cliente cliente;
	@Temporal(TemporalType.DATE)
	private final Date fecha;
	private final BigDecimal monto;
	private final String numero;
	@Enumerated(EnumType.STRING)
	private final TipoFactura tipoFactura;
	@OneToMany(cascade = { CascadeType.PERSIST })
	private final List<LineaDeFacturaProductoInmutable> lineasDeFactura;
	@OneToMany(cascade = { CascadeType.PERSIST })
	private final List<LineaDeFacturaDescuento> lineasDeDescuento;
	private final boolean consumidorFinal;
	
	protected FacturaInmutable(final Cliente cliente, final Date fecha, final BigDecimal monto, final String numero, final TipoFactura tipoFactura, final List<LineaDeFacturaProductoInmutable> lineasDeFactura, final List<LineaDeFacturaDescuento> lineasDeDescuento) {
		if (fecha == null) throw new NullPointerException("La fecha no puede ser null" + FacturaInmutable.class);
		if (monto == null) throw new NullPointerException("El monto no puede ser null" + FacturaInmutable.class);
		if (numero == null) throw new NullPointerException("El numero no puede ser null" + FacturaInmutable.class);
		if (tipoFactura == null) throw new NullPointerException("El tipo de factura no puede ser null" + FacturaInmutable.class);
		if (lineasDeFactura.size() == 0) throw new IllegalArgumentException("La lista de lineas de factura esta vacia" + FacturaInmutable.class);
		this.cliente = cliente;
		if (cliente == null) {
			consumidorFinal = true;
		} else {
			consumidorFinal = false;			
		}
		
		this.fecha = (Date) fecha.clone();
		this.monto = monto;
		this.numero = numero;
		this.tipoFactura = tipoFactura;
		this.lineasDeFactura = Collections.unmodifiableList(lineasDeFactura);
		this.lineasDeDescuento = Collections.unmodifiableList(lineasDeDescuento);
		
	}
	
	public Date getFecha() {
		return (Date)fecha.clone();
	}

	public BigDecimal getMonto() {
		return monto;
	}

	public String getNumero() {
		return numero;
	}

	public TipoFactura getTipoFactura() {
		return tipoFactura;
	}

	public List<LineaDeFacturaProductoInmutable> getLineasDeFactura() {
		return Collections.unmodifiableList(lineasDeFactura);
	}

	public List<LineaDeFacturaDescuento> getLineasDeDescuento() {
		return Collections.unmodifiableList(lineasDeDescuento);
	}
	
	public Cliente getCliente() {
		return this.cliente;
	}
	
	public BigDecimal getMontoIVA() {
		BigDecimal suma = BigDecimal.ZERO;
		for (LineaDeFacturaProductoInmutable linea : lineasDeFactura) {
			suma = suma.add(linea.getMontoIVA()).setScale(3,RoundingMode.HALF_EVEN);
		}
		return suma;
	}
	
	public BigDecimal getMontoDescuento() {
		BigDecimal suma = BigDecimal.ZERO;
		for (LineaDeFacturaDescuento linea : lineasDeDescuento) {
			suma = suma.add(linea.getMontoDescuento()).setScale(3,RoundingMode.HALF_EVEN);
		}
		return suma;
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof FacturaInmutable))
			return false;
		FacturaInmutable fac = (FacturaInmutable) obj;
		if (!this.getNumero().equals(fac.getNumero())) return false;
		if (!this.getMonto().equals(fac.getMonto())) return false;
		if (!this.getCliente().equals(fac.getCliente())) return false;
		if (!this.getFecha().equals(fac.getFecha())) return false;
		if (!this.getTipoFactura().equals(fac.getTipoFactura()));
		if (!this.getLineasDeFactura().equals(fac.getLineasDeFactura())) return false;
		if (!this.getLineasDeDescuento().equals(fac.getLineasDeDescuento())) return false;		
		return true;
	}

	@Override
	public int hashCode() {
		int result = 37;
		result = 37 * result + this.getNumero().hashCode();
		result = 37 * result + this.getMonto().hashCode();
		result = 37 * result + this.getCliente().hashCode();
		result = 37 * result + this.getFecha().hashCode();
		result = 37 * result + this.getTipoFactura().hashCode();
		result = 37 * result + this.getLineasDeFactura().hashCode();
		result = 37 * result + this.getLineasDeDescuento().hashCode();
		return result;
	}
	
	@Override
	public int compareTo(FacturaInmutable o) {
		return this.getFecha().compareTo(o.getFecha());
	}

	public boolean isConsumidorFinal() {
		return consumidorFinal;
	}
}
